﻿unit u_filter_components;

{$I ..\DLCompilers.inc}
{$I ..\extends.inc}
{$IFDEF FPC}
{$mode Delphi}
{$ENDIF}

interface

uses
{$IFDEF FPC}
 LCLIntf, LCLType,
 lmessages,
 {$ELSE}
 Windows, Mask, DBTables,
{$ENDIF}
 u_framework_components,
{$IFDEF VERSIONS}
  fonctions_version,
{$ENDIF}
  Graphics, Controls, Classes, Dialogs, Messages,
  Buttons, Forms, DBCtrls,
  StdCtrls, SysUtils,
  TypInfo, Variants, u_extcomponent,
  Grids, ExtCtrls,
  fonctions_components,
  u_extradios,
  fonctions_erreurs;
const
   CST_FILTERED_FIELD = 'FilteredField';
   CST_FILTERED_CONTROL = 'FilteredControl';
{$IFDEF VERSIONS}
    gVer_filter_components : T_Version = ( Component : 'Composants de filtrage' ;
                                               FileUnit : 'u_filter_components' ;
                                               Owner : 'Matthieu Giroux' ;
                                               Comment : 'Composants d''interactivité de U_CustomFrameWork.' ;
                                               BugsStory :  '0.9.0.0 : Filter components for XML Frames import field.';
                                               UnitType : 3 ;
                                               Major : 0 ; Minor : 9 ; Release : 0 ; Build : 0 );

{$ENDIF}
type
   IFilterComponent = interface
   ['{12CBE27F-94C1-4A3D-B94F-FE57D36207D5}'] // GUID nécessaire pour l'opération de cast
       procedure SetFilterToComponent;
       // Need to have property FilteredComponent
     End;

   { TFWFilterEdit }

   TFWFilterEdit = class ( TFWEdit, IFilterComponent )
      private
       FFilteredControl : TControl;
       FFilteredField : String;
      protected
      public
       constructor Create ( AOwner : TComponent ); override;
       procedure Change; override;
       procedure SetFilterToComponent;
      published
       property FilteredControl: TControl read FFilteredControl write FFilteredControl;
       property FilteredField     : String     read FFilteredField write FFilteredField;
     End;


    { TFWFilterSpinEdit }

    TFWFilterSpinEdit = class ( TFWSpinEdit, IFilterComponent )
       private
        FFilteredControl : TControl;
        FFilteredField : String;
       protected
       public
        constructor Create ( AOwner : TComponent ); override;
        procedure Change; override;
        procedure SetFilterToComponent;
       published
        property FilteredControl: TControl read FFilteredControl write FFilteredControl;
        property FilteredField     : String     read FFilteredField write FFilteredField;
      End;

     { TFWFilterDateEdit }

     TFWFilterDateEdit = class ( TFWDateEdit, IFilterComponent )
        private
         FFilteredControl : TControl;
         FFilteredField : String;
        protected
        public
         constructor Create ( AOwner : TComponent ); override;
         procedure EditChange; override;
         procedure SetFilterToComponent;
        published
         property FilteredControl: TControl read FFilteredControl write FFilteredControl;
         property FilteredField     : String     read FFilteredField write FFilteredField;
       End;

   { TFWFilterComboBox }

   TFWFilterComboBox = class ( TFWComboBox, IFilterComponent )
      private
       FFilteredControl : TControl;
       FFilteredField : String;
      protected
      public
       constructor Create ( AOwner : TComponent ); override;
       procedure Change; override;
       procedure SetFilterToComponent;
      published
       property FilteredControl: TControl read FFilteredControl write FFilteredControl;
       property FilteredField     : String     read FFilteredField write FFilteredField;
     End;

   { TFWFilterGrid }

   TFWFilterGrid = class ( TFWGrid, IFilterComponent )
      private
       FFilteredControl : TControl;
       FFilteredField : String;
      protected
      public
       constructor Create ( AOwner : TComponent ); override;
       function  GetEditText(aCol, aRow: Integer): string; override;
       procedure SetFilterToComponent;
      published
       property FilteredControl: TControl read FFilteredControl write FFilteredControl;
       property FilteredField     : String     read FFilteredField write FFilteredField;
     End;



    { TFWFilterDateTimePicker }

    TFWFilterDateTimePicker = class ( TFWDateTimePicker, IFilterComponent )
       private
        FFilteredControl : TControl;
        FFilteredField : String;
       protected
       public
        constructor Create ( AOwner : TComponent ); override;
        procedure Change; override;
        procedure SetFilterToComponent;
       published
        property FilteredControl: TControl read FFilteredControl write FFilteredControl;
        property FilteredField     : String     read FFilteredField write FFilteredField;
      End;

    { TFWFilterRadioGroup }

    TFWFilterRadioGroup = class ( TExtRadioGroup, IFilterComponent )
       private
        FFilteredControl : TControl;
        FFilteredField : String;
       protected
       public
        constructor Create ( AOwner : TComponent ); override;
        procedure CheckItemIndexChanged; override;
        procedure SetFilterToComponent;
       published
        property FilteredControl: TControl read FFilteredControl write FFilteredControl;
        property FilteredField     : String     read FFilteredField write FFilteredField;
      End;

 function fb_SetFilterToComponent ( const AFilteredComponent : TComponent ; const AFilteredField : String; const Avalue : Integer ): Boolean; overload;
 function fb_SetFilterToComponent ( const AFilteredComponent : TComponent ; const AFilteredField : String; const Avalue : String ): Boolean; overload;
 function fb_SetFilterToComponent ( const AFilteredComponent : TComponent ; const AFilteredField : String; const Avalue : Double ): Boolean; overload;
 function fb_SetFilterToComponent ( const AFilteredComponent : TComponent ; const AFilteredField : String; const Avalue : TDateTime ): Boolean; overload;
 function fb_SetFilteredFieldToComponent ( const AFilteredComponent : TComponent ; const AFilter : String ): Boolean;

implementation

uses fonctions_db,fonctions_proprietes, U_GroupView,DB;

{ TFWFilterSpinEdit }

procedure TFWFilterSpinEdit.SetFilterToComponent;
begin
  fb_SetFilterToComponent ( FFilteredControl, FFilteredField, Value );
end;

constructor TFWFilterSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterSpinEdit.Change;
begin
  inherited Change;
  SetFilterToComponent;
end;

{ TFWFilterEdit }

procedure TFWFilterEdit.SetFilterToComponent;
begin
  fb_SetFilterToComponent ( FFilteredControl, FFilteredField, Text );

end;

constructor TFWFilterEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterEdit.Change;
begin
  inherited Change;
  SetFilterToComponent;
end;

{ TFWFilterDateEdit }

procedure TFWFilterDateEdit.SetFilterToComponent;
begin
  fb_SetFilterToComponent ( FFilteredControl, FFilteredField, Date );

end;

constructor TFWFilterDateEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterDateEdit.EditChange;
begin
  inherited Changed;
  SetFilterToComponent;
end;

{ TFWFilterComboBox }

procedure TFWFilterComboBox.SetFilterToComponent;
begin
  if ItemIndex > -1 Then
    fb_SetFilterToComponent ( FFilteredControl, FFilteredField, Items[ItemIndex] );
end;

constructor TFWFilterComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterComboBox.Change;
begin
  inherited Change;
  SetFilterToComponent;
end;

{ TFWFilterGrid }

procedure TFWFilterGrid.SetFilterToComponent;
begin
  fb_SetFilterToComponent ( FFilteredControl, FFilteredField, Text );
end;

constructor TFWFilterGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TFWFilterGrid.GetEditText(aCol, aRow: Integer): string;
begin
  Result := inherited;
  SetFilterToComponent;
end;



{ TFWFilterDateTimePicker }

procedure TFWFilterDateTimePicker.SetFilterToComponent;
begin
  fb_SetFilterToComponent ( FFilteredControl, FFilteredField, DateTime );

end;

constructor TFWFilterDateTimePicker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterDateTimePicker.Change;
begin
  inherited Changed;
  SetFilterToComponent;
end;



{ TFWFilterRadioGroup }

procedure TFWFilterRadioGroup.SetFilterToComponent;
begin
  if ItemIndex > -1 Then
    fb_SetFilterToComponent ( FFilteredControl, FFilteredField, ItemIndex );

end;

constructor TFWFilterRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TFWFilterRadioGroup.CheckItemIndexChanged;
begin
  inherited Changed;
  SetFilterToComponent;
end;


function fb_SetFilterToComponent ( const AFilteredComponent : TComponent; const AFilteredField : String; const Avalue : Integer ): Boolean;
Begin
  Result := fb_SetFilteredFieldToComponent ( AFilteredComponent, AFilteredField + ' = ' + IntToStr(Avalue));
end;

function fb_SetFilterToComponent(const AFilteredComponent: TComponent; const AFilteredField : String;
  const Avalue: String): Boolean;
begin
  Result := fb_SetFilteredFieldToComponent ( AFilteredComponent, AFilteredField + ' =' + fs_AddDBString (Avalue));

end;

function fb_SetFilterToComponent(const AFilteredComponent: TComponent; const AFilteredField : String;
  const Avalue: Double): Boolean;
begin
  Result := fb_SetFilteredFieldToComponent ( AFilteredComponent, AFilteredField + ' = ' + FloatToStr(Avalue));
end;

function fb_SetFilterToComponent(const AFilteredComponent: TComponent; const AFilteredField : String;
  const Avalue: TDateTime): Boolean;
begin
  Result := fb_SetFilteredFieldToComponent ( AFilteredComponent, AFilteredField + ' = ' + FormatDateTime(CST_DateTimeFormat,Avalue));
end;

function fb_SetFilteredFieldToComponent ( const AFilteredComponent : TComponent ; const AFilter : String ): Boolean;
var ADatasource : TObject;
    ADataset    : TDataset;
Begin
  Result:=False;
  ADatasource :=nil;

  if   IsPublishedProp ( AFilteredComponent, CST_DBPROPERTY_LOOKUPSOURCE)
  and  PropIsType      ( AFilteredComponent, CST_DBPROPERTY_LOOKUPSOURCE , tkClass)
  then ADatasource := GetObjectProp ( AFilteredComponent, CST_DBPROPERTY_LOOKUPSOURCE);

  if   IsPublishedProp ( AFilteredComponent, CST_DBPROPERTY_SEARCHSOURCE)
  and  PropIsType      ( AFilteredComponent, CST_DBPROPERTY_SEARCHSOURCE , tkClass)
  then ADatasource := GetObjectProp ( AFilteredComponent, CST_DBPROPERTY_SEARCHSOURCE);

  if ( ADatasource = nil )
  and ( AFilteredComponent is TDBGroupView ) Then
   ADatasource := ( AFilteredComponent as TDBGroupView ).Datasource;

  if ( ADatasource = nil )
   Then Exit;
  ADataset:=(ADatasource as TDatasource).DataSet;
  try
    ADataset.Filter:=AFilter;
    ADataset.Filtered:=True;
    Result:=True;
    if AFilteredComponent is TDBGroupView then
     Begin
       ADataset:=(AFilteredComponent as TDBGroupView).DataListOpposite.Datasource.DataSet;
       ADataset.Filter:=AFilter;
       ADataset.Filtered:=True;
     end;
  finally
  end;
end;

{$IFDEF VERSIONS}
initialization
  // Gestion de version
  p_ConcatVersion(gVer_filter_components);
{$ENDIF}
end.

